{ SMReport Designer

  Copyright (C) 1998-2003, written by Mike Shkolnik, Scalabium Software
  FIDOnet: 2:463/106.14
  E-Mail:  smreport@scalabium.com
           mshkolnik@scalabium.com
           mshkolnik@yahoo.com
  WEB: http://www.scalabium.com
       http://www.geocities.com/mshkolnik
  tel: 380-/44/-552-10-29

  In this unit I create a TSMRCheckBox component for
  demonstation of technology for custom control creation
  in SMReport Designer.
}
unit SMRCheck;

interface

uses
  Windows, Classes, graphics, StdCtrls, Controls,
  DB, SMRBase;

const
  {range for custom constants is:
   for lnEnglish - 11000..12999
   for lnRussian - 13000..14999
   for lnUkrainian - 15000..16999
   for lnGerman - 17000..18999
   for lnFrance - 19000..20999
   for lnSpanish - 21000..22999
   for lnItalian - 23000..24999
   for lnPortuguese - 25000..26999
  }
  smrcNewCheckBox = 2501;
  smrcDesignerBtnCheckBox = 2502;
  smrcCheckStyle = 2503;
  smrcCheckStandard = 2504;
  smrcCheckCheck = 2505;
  smrcCheckCross = 2506;

type
  { checkbox on the band }
  TSMRCheckStyle = (csStandard, csCheck, csCross);

  TSMRCheckBox = class(TSMRControl)
  private
    FExpression: string;
    FState: TCheckBoxState;

    FCheckWidth: Integer;
    FCheckHeight: Integer;

    FCheckBitmap: TBitmap;

    FStyle: TSMRCheckStyle;
  protected
    { Protected declarations }
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;

    function Draw(ACanvas: TCanvas; var ARect: TRect; intBandHeight: Integer; Flags: Word): Integer; override;
    function EditProperties(ShowPrevForm: Boolean): Boolean; override;
  published
    { Published declarations }
    property State: TCheckBoxState read FState write FState;
    property Expression: string read FExpression write FExpression;
    property Style: TSMRCheckStyle read FStyle write FStyle default csStandard;
  end;

implementation
{$R *.RES}
{$R SMRChk.RES}

uses SMRCnst, SMRUtils, SMRChEd, forms;

{ TSMRCheckBox }
constructor TSMRCheckBox.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);

  DisplayName := GetString(FSMReport.GetLanguageShift, smrcNewCheckBox);

  FStyle := csStandard;
  FCheckBitmap := TBitmap.Create;
  FCheckBitmap.Transparent := True;

  with TBitmap.Create do
    try
      Handle := LoadBitmap(0, PChar(32759));
      FCheckWidth := Width div 4;
      FCheckHeight := Height div 3;
    finally
      Free;
    end;
end;

destructor TSMRCheckBox.Destroy;
begin
  FCheckBitmap.Free;

  inherited;
end;

function TSMRCheckBox.Draw(ACanvas: TCanvas; var ARect: TRect; intBandHeight: Integer; Flags: Word): Integer;
var
  DS: TDataSet;
  fld: TField;
  AState: TCheckBoxState;
  DrawState: Integer;
  DrawRect: TRect;
begin
  Result := inherited Draw(ACanvas, ARect, intBandHeight, Flags);

  if (Flags and DT_CALCRECT <> 0) then Exit;

  if (Expression = '') then
    AState := State
  else
  begin
    AState := cbUnChecked;

    DS := GetDSByName(FSMReport.GetDataManager, GetDSName(Expression));
    if Assigned(DS) then
      try
        fld := DS.FieldByName(GetFieldName(Expression));
        if (fld.DataType = ftBoolean) then
          if fld.IsNull then
            AState := cbGrayed
          else
            if fld.AsBoolean then
              AState := cbChecked;
       except
       end;
  end;

  ACanvas.Brush.Color := BackColor;
  case Alignment of
    taRightJustify: begin
                      DrawRect.Left := ARect.Right - FCheckWidth;
                      DrawRect.Right := ARect.Right;
                    end;
    taCenter: begin
                DrawRect.Left := ARect.Left + (ARect.Right - ARect.Left - FCheckWidth) div 2;
                DrawRect.Right := DrawRect.Left + FCheckWidth;
              end;
  else // taLeftJustify
    DrawRect.Left := ARect.Left;
    DrawRect.Right := DrawRect.Left + FCheckWidth;
  end;
  DrawRect.Top := ARect.Top + (ARect.Bottom - ARect.Top - FCheckWidth) div 2;
  DrawRect.Bottom := DrawRect.Top + FCheckHeight;

  case FStyle of
    csCheck: begin
               case AState of
                 cbChecked: FCheckBitmap.Handle := LoadBitmap(hInstance, PChar('SMR_CHECK_ON'));
                 cbUnchecked: FCheckBitmap.Handle := LoadBitmap(hInstance, PChar('SMR_CHECK_OFF'));
               else // cbGrayed
                 FCheckBitmap.Handle := LoadBitmap(hInstance, PChar('SMR_CHECK_GRAY'));
               end;
               ACanvas.Draw(DrawRect.Left, DrawRect.Top, FCheckBitmap)
             end;
    csCross: begin
               case AState of
                 cbChecked: FCheckBitmap.Handle := LoadBitmap(hInstance, PChar('SMR_CROSS_ON'));
                 cbUnchecked: FCheckBitmap.Handle := LoadBitmap(hInstance, PChar('SMR_CROSS_OFF'));
               else // cbGrayed
                 FCheckBitmap.Handle := LoadBitmap(hInstance, PChar('SMR_CROSS_GRAY'));
               end;
               ACanvas.Draw(DrawRect.Left, DrawRect.Top, FCheckBitmap)
             end;
  else //csStandard
    case AState of
      cbChecked: DrawState := DFCS_BUTTONCHECK or DFCS_CHECKED;
      cbUnchecked: DrawState := DFCS_BUTTONCHECK;
      else // cbGrayed
        DrawState := DFCS_BUTTON3STATE or DFCS_CHECKED;
    end;

    DrawFrameControl(ACanvas.Handle, DrawRect, DFC_BUTTON, DrawState);
  end;
end;

function TSMRCheckBox.EditProperties(ShowPrevForm: Boolean): Boolean;
begin
  Result := inherited EditProperties(False);

  if ShowPrevForm then
  begin
    with TfrmEditCheck.Create(Application) do
      try
        LangShift := FSMReport.GetLanguageShift;
        FillStringsFromRes;

        FSMReport.AssignDataSets(cbExpression.Items);
        PrintWhen := Self.SMPrintWhen;

        FSMReport.AssignGroups(True, frmPrintWhen.cbGroups.Items);
        if (frmPrintWhen.cbGroups.Items.Count = 0) then
          frmPrintWhen.cbGroupChange.Enabled := False;

        if (Expression = '') then
        begin
          rbValue.Checked := True;
          cbState.ItemIndex := Ord(State)
        end
        else
        begin
          rbField.Checked := True;
          cbExpression.ItemIndex := cbExpression.Items.IndexOf(Expression);
        end;

        cbStyle.ItemIndex := Ord(Style);
        rgPosition.ItemIndex := Ord(ObjectPosition);
        cbTransparent.Checked := Transparent;
        memoComment.Lines.Assign(Comments);

        Result := (ShowModal = mrOk);
        if Result then
        begin
          Transparent := cbTransparent.Checked;

          if rbValue.Checked then
          begin
            Expression := '';
            State := TCheckBoxState(cbState.ItemIndex);
            DisplayName := cbState.Items[cbState.ItemIndex];
          end
          else
          begin
            Expression := cbExpression.Items[cbExpression.ItemIndex];
            State := cbUnChecked;
            DisplayName := Expression;
          end;
          Style := TSMRCheckStyle(cbStyle.ItemIndex);

          SMPrintWhen := PrintWhen;

          ObjectPosition := TSMRObjectPosition(rgPosition.ItemIndex);
          Comments.Assign(memoComment.Lines);

          Invalidate;
        end;
      finally
        Free
      end;
  end;
end;


initialization
  RegisterSMRControl(TSMRCheckBox, smrcDesignerBtnCheckBox, '', hInstance);

finalization
  UnRegisterSMRControl(TSMRCheckBox);

end.
