unit Main;

interface

uses
  Classes, StdCtrls, Controls, ExtCtrls, Forms,
  PrintSM, SMRBase, SMRAuto, Db, DBTables;

type
  TfrmMain = class(TForm)
    rgSelectReport: TRadioGroup;
    gbDescription: TGroupBox;
    SMReport: TSMReport;
    dbMain: TDatabase;
    qryExample: TQuery;
    btnPreview: TButton;
    btnDesign: TButton;
    btnPrint: TButton;
    btnExit: TButton;
    smrAutoDS: TSMPrintData;
    smrAutoText: TSMPrintText;
    lblLogo1: TLabel;
    lblLogo2: TLabel;
    lblLogo3: TLabel;
    lblLogo4: TLabel;
    lblLogo5: TLabel;
    memoDescription: TMemo;
    smrComposite: TSMRCompositeReport;
    rgLanguage: TRadioGroup;
    cbCustomPreview: TCheckBox;
    procedure rgSelectReportClick(Sender: TObject);
    procedure btnPreviewClick(Sender: TObject);
    procedure btnExitClick(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure lblLogo4Click(Sender: TObject);
    procedure rgLanguageClick(Sender: TObject);
  private
    { Private declarations }
    function GetReport: TSMPrintBaseComponent;
    procedure CustomPreviewEvent(Sender: TObject);
  public
    { Public declarations }
  end;

var
  frmMain: TfrmMain;

implementation

{$R *.DFM}
uses SMDesign, SysUtils, ShellAPI, Windows, CustPrev;

procedure TfrmMain.rgSelectReportClick(Sender: TObject);
var strPath: string;
begin
  strPath := ExtractFilePath(Application.ExeName);
  case rgSelectReport.ItemIndex of
    0: begin
         qryExample.SQL.Text := 'SELECT * FROM CUSTOMER ORDER BY COMPANY';
         SMReport.LoadReport(strPath + '\SimpleLst.SMR');
       end;
    1: begin
         qryExample.SQL.Text := 'SELECT * FROM CUSTOMER ORDER BY COMPANY';
         SMReport.LoadReport(strPath + '\ColList.SMR');
       end;
    2: begin
         qryExample.SQL.Text := 'SELECT CUSTOMER.*, ORDERS.* FROM ORDERS, CUSTOMER WHERE (ORDERS.CustNo = CUSTOMER.CustNo) ORDER BY CUSTOMER.COMPANY';
         SMReport.LoadReport(strPath + '\SimpleGrp.SMR');
       end;
    3: begin
         qryExample.SQL.Text := 'SELECT CUSTOMER.*, ORDERS.*, ITEMS.*, PARTS.*, ' +
                                '       ITEMS.QTY*PARTS.LISTPRICE AS TOTAL ' +
                                'FROM PARTS, ITEMS, ORDERS, CUSTOMER ' +
                                'WHERE (ORDERS.CustNo = CUSTOMER.CustNo) ' +
                                '  AND (ITEMS.OrderNo = ORDERS.OrderNo) ' +
                                '  AND (PARTS.PartNo = ITEMS.PartNo) ' +
                                'ORDER BY CUSTOMER.COMPANY, ORDERS.SaleDate, ITEMS.ItemNo';
         SMReport.LoadReport(strPath + '\ManyGrp.SMR');
       end;
    4: begin
         qryExample.SQL.Text := 'SELECT * FROM BIOLIFE';
         SMReport.LoadReport(strPath + '\SimplePic.SMR');
       end;
    5: memoDescription.Lines.Assign(smrAutoDS.SMProperty.Comments);
    6: memoDescription.Lines.Assign(smrAutoText.SMProperty.Comments);
    7: begin
         memoDescription.Lines.Assign(smrComposite.SMProperty.Comments);
         smrComposite.ClearReports;
         smrComposite.AddReport(SMReport);
         smrComposite.AddReport(smrAutoDS);
       end;
  end;
  if (rgSelectReport.ItemIndex < 5) then
  begin
    memoDescription.Lines.Assign(SMReport.SMProperty.Comments);
    btnDesign.Enabled := True
  end
  else
    btnDesign.Enabled := False
end;

procedure TfrmMain.btnPreviewClick(Sender: TObject);
begin
  qryExample.Open;
  with GetReport do
    case (Sender as TControl).Tag of
      1: DesignReport(SMReport);
      2: begin
           if cbCustomPreview.Checked then
             OnPreview := CustomPreviewEvent
           else
             OnPreview := nil;
           Preview;
         end;
      3: Print;
    end;
//  qryExample.Close;
end;

procedure TfrmMain.btnExitClick(Sender: TObject);
begin
  Close
end;

procedure TfrmMain.FormActivate(Sender: TObject);
begin
  rgSelectReportClick(Sender)
end;

function TfrmMain.GetReport: TSMPrintBaseComponent;
begin
  case rgSelectReport.ItemIndex of
    5: Result := smrAutoDS;
    6: begin
         smrAutoText.FileName := '';
         Result := smrAutoText;
       end;
    7: Result := smrComposite;
  else
    Result := SMReport;
  end;
end;


procedure TfrmMain.lblLogo4Click(Sender: TObject);
begin
  ShellExecute(0, 'open', PChar((Sender as TLabel).Caption), nil, nil, SW_SHOWNORMAL);
end;

procedure TfrmMain.rgLanguageClick(Sender: TObject);
begin
  SMReport.Language.AutoDetect := (rgLanguage.ItemIndex < 1);
  smrAutoDS.Language.AutoDetect := SMReport.Language.AutoDetect;
  smrAutoText.Language.AutoDetect := SMReport.Language.AutoDetect;
  smrComposite.Language.AutoDetect := SMReport.Language.AutoDetect;

  if (rgLanguage.ItemIndex > 0) then
  begin
    SMReport.Language.LanguageID := TSMRLangID(rgLanguage.ItemIndex - 1);
    smrAutoDS.Language.LanguageID := SMReport.Language.LanguageID;
    smrAutoText.Language.LanguageID := SMReport.Language.LanguageID;
    smrComposite.Language.LanguageID := SMReport.Language.LanguageID;
  end
end;

procedure TfrmMain.CustomPreviewEvent(Sender: TObject);
begin
  with TfrmCustomPreview.Create(Application) do
  try
    SMRPreview.Report := (Sender as TSMPrintBaseComponent);
    ShowModal
  finally
    Free
  end;
end;

end.
